package com.qmth.distributed.print.business.service.impl;

import cn.hutool.core.lang.UUID;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.config.DictionaryConfig;
import com.qmth.distributed.print.business.entity.BasicAttachment;
import com.qmth.distributed.print.business.mapper.BasicAttachmentMapper;
import com.qmth.distributed.print.business.service.BasicAttachmentService;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.business.enums.StorageLevelEnum;
import com.qmth.distributed.print.business.enums.StorageTypeEnum;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.StringJoiner;

/**
 * <p>
 * 所有附件记录表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicAttachmentServiceImpl extends ServiceImpl<BasicAttachmentMapper, BasicAttachment> implements BasicAttachmentService {

    @Autowired
    private DictionaryConfig dictionaryConfig;

    @Override
    public BasicAttachment save(MultipartFile file, String md5, Long schoolId, Long userId) {
        try {
            if (file == null) {
                throw ExceptionResultEnum.ERROR.exception("文件为空");
            }
            if (StringUtils.isBlank(md5)) {
                throw ExceptionResultEnum.ERROR.exception("md5为空");
            }
            // 文件名
            String fileName = file.getOriginalFilename();
            int lastIndex = fileName.lastIndexOf(".");
            // 文件类型
            String type = fileName.substring(lastIndex);
            List<String> attachmentType = dictionaryConfig.sysDomain().getAttachmentType();
            if (attachmentType != null && attachmentType.size() > 0) {
                if (!attachmentType.contains(type)) {
                    throw ExceptionResultEnum.ERROR.exception("文件格式不支持");
                }
            }

            // 文件大小
            long size = file.getSize();
            if (size > 200 * 1024 * 1024) {
                throw ExceptionResultEnum.ERROR.exception("文件大小不能超过200M");
            }

            // md5
            String fileMd5 = DigestUtils.md5Hex(file.getBytes());
            if (!md5.equals(fileMd5)) {
                throw ExceptionResultEnum.ERROR.exception("md5不一致，文件可能损坏");
            }

            // 保存
            BasicAttachment attachment = new BasicAttachment();

            // 保存文件
            boolean isOss = dictionaryConfig.sysDomain().isOss();
            // 文件保存路径
            LocalDateTime localDateTime = LocalDateTime.now();
            String serverUpload = dictionaryConfig.sysDomain().getServerUpload();
            StringJoiner sj = new StringJoiner(File.separator)
                    .add(serverUpload)
                    .add(String.valueOf(schoolId))
                    .add(String.valueOf(localDateTime.getYear()))
                    .add(String.valueOf(localDateTime.getMonthValue()))
                    .add(String.valueOf(localDateTime.getDayOfMonth()));
            if (isOss) {
                // todo
            } else {
                File destFolder = new File(sj.toString());
                if (!destFolder.exists()) {
                    destFolder.mkdirs();
                }
                File newFile = new File(sj.toString(), UUID.fastUUID() + type);
                FileUtils.copyInputStreamToFile(file.getInputStream(), newFile);
                attachment.setPath(newFile.getPath());
            }

            attachment.setName(file.getOriginalFilename());
            attachment.setType(type);
            attachment.setSize(Math.toIntExact(size));
            attachment.setMd5(md5);
            attachment.setLevel(StorageLevelEnum.L.name());
            attachment.setStorage(isOss ? StorageTypeEnum.OSS.name() : StorageTypeEnum.LOCAL.name());
            attachment.setCreateId(userId);
            attachment.setCreateTime(System.currentTimeMillis());

            this.save(attachment);
            return attachment;
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
        }
        return null;
    }
}
