package com.qmth.distributed.print.api;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.distributed.print.business.bean.ApiUser;
import com.qmth.distributed.print.business.bean.dto.BlurryUserDto;
import com.qmth.distributed.print.business.bean.dto.ExamTaskDto;
import com.qmth.distributed.print.business.bean.dto.ExamTaskImportDto;
import com.qmth.distributed.print.business.bean.dto.ReviewLogDto;
import com.qmth.distributed.print.business.bean.query.BlurryParam;
import com.qmth.distributed.print.business.bean.query.ExamTaskParam;
import com.qmth.distributed.print.business.entity.ExamTask;
import com.qmth.distributed.print.business.entity.ExamTaskReviewLog;
import com.qmth.distributed.print.business.service.ExamTaskReviewLogService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.business.util.RequestUtils;
import com.qmth.distributed.print.common.util.Result;
import com.qmth.distributed.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 命题任务 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "命题任务Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.exam}/task")
@Aac(auth = BOOL.FALSE)
public class ExamTaskController {

    @Autowired
    private ExamTaskService examTaskService;

    @Autowired
    private ExamTaskReviewLogService examTaskReviewLogService;

    /**
     * 查询
     * @param request
     * @param param
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Result list(HttpServletRequest request, ExamTaskParam param){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        param.setSchoolId(apiUser.getSchoolId());
        IPage<ExamTaskDto> examTasks = examTaskService.list(param);
        return ResultUtil.ok(examTasks);
    }

    /**
     * 试卷编号模糊查询
     * @param param
     * @return
     */
    @ApiOperation(value = "试卷编号模糊查询")
    @RequestMapping(value = "/paper_number_query", method = RequestMethod.GET)
    public Result paperNumberQuery(HttpServletRequest request, BlurryParam param){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        param.setSchoolId(apiUser.getSchoolId());
        List<String> list = examTaskService.listPaperNumber(param);
        return ResultUtil.ok(list);
    }

    /**
     * 命题老师模糊查询
     * @param param
     * @return
     */
    @ApiOperation(value = "命题老师模糊查询")
    @RequestMapping(value = "/user_query", method = RequestMethod.GET)
    public Result userQuery(HttpServletRequest request, BlurryParam param){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        param.setSchoolId(apiUser.getSchoolId());
        List<BlurryUserDto> list = examTaskService.listUser(param);
        return ResultUtil.ok(list);
    }

    /**
     * 更改/指派命题老师
     * @param request
     * @param examTask
     * @return
     */
    @ApiOperation(value = "更改/指派命题老师")
    @RequestMapping(value = "/assign_user", method = RequestMethod.POST)
    public Result assignUser(HttpServletRequest request, @RequestBody ExamTask examTask){
        boolean isSuccess = examTaskService.assignUser(examTask);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 启用/禁用
     * @param request
     * @param examTask
     * @return
     */
    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable", method = RequestMethod.POST)
    public Result enable(HttpServletRequest request, @RequestBody ExamTask examTask){
        boolean isSuccess = examTaskService.enable(examTask);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 新建
     * @param request
     * @param examTask
     * @return
     */
    @ApiOperation(value = "新建")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(HttpServletRequest request, @RequestBody ExamTask examTask){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        examTask.setSchoolId(apiUser.getSchoolId());
        examTask.setCreateId(apiUser.getUserData().getId());
        examTask.setCreateTime(System.currentTimeMillis());
        boolean isSuccess = examTaskService.saveExamTask(examTask);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 文件导入
     * @param request
     * @param file
     * @param md5
     * @return
     */
    @ApiOperation(value = "文件导入")
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    public Result importFile(HttpServletRequest request, MultipartFile file, String md5){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        List<ExamTaskImportDto> list = examTaskService.importFile(file, md5, apiUser);
        return ResultUtil.ok(list);
    }

    /**
     * 批量保存
     * @param request
     * @param task
     * @return
     */
    @ApiOperation(value = "批量保存")
    @RequestMapping(value = "/save_batch", method = RequestMethod.POST)
    public Result saveBatch(HttpServletRequest request, ExamTask task){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        task.setSchoolId(apiUser.getSchoolId());
        task.setCreateId(apiUser.getUserData().getId());
        boolean isSuccess = examTaskService.saveBatch(task);
        return ResultUtil.ok(isSuccess);
    }


    @ApiOperation(value = "审核历史")
    @RequestMapping(value = "/review_list", method = RequestMethod.GET)
    public Result reviewList(HttpServletRequest request, ExamTaskReviewLog reviewLog){
        List<ReviewLogDto> list = examTaskReviewLogService.list(reviewLog);
        return ResultUtil.ok(list);
    }
}

